// This is -*- C++ -*-
// $Id: statsdb.cc,v 1.1 1998/09/29 23:16:53 hp Exp $

/* statsdb.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Havoc Pennington <hp@emccta.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "StatsDB.h"
#include <sys/time.h>
#include <unistd.h>

// Compare with "dataset1.cc" for simplicity.

int main(int argc, char** argv)
{
  DataSet ds1;
  DataSet ds2;

  // Make up data
  double d = 0.02;
  double end = d+800.0;
  int j = 0;
  while ( d < end ) {
    int r = rand();
    double v;
    if (r % 2 == 0) v = d + (double)r/(double)(RAND_MAX/12);
    else v = d - (double)r/(double)(RAND_MAX/15);
    ds1.add(v);
    ds2.add(v*rand()/500.0);
    d += (200.73/(j+1));
    ++j;
  }

  cout << endl;

  StatsDB sdb;

  cout << "Stats for two data sets, N = " << ds1.size() << ", N = " << ds2.size() << endl;

  StatsDB::const_iterator i = sdb.single_begin();
  while ( i != sdb.single_end() ) {
    cout << "1 - " << *i << ": " << sdb.statistic(*i,ds1) << endl;
    cout << "2 - " << *i << ": " << sdb.statistic(*i,ds2) << endl;
    ++i;
  }
  
  i = sdb.double_begin();
  while ( i != sdb.double_end() ) {
    cout << *i << ": " << sdb.statistic(*i,ds1,ds2) << endl;
    ++i;
  }

  size_t q = 1;
  double times[5];

  while ( q <= 5 ) {
    size_t repetitions = 5000*q;
    cout << endl << "Speed test: recomputing same stats " << repetitions << " times via StatsDB: ";
    
    struct timeval start;
    struct timeval stop;
    if (gettimeofday(&start,0) == -1) cerr << "couldn't get time" << endl;
    
    size_t k = 0;
    while ( k < repetitions ) {
      double result;
      
      if (k%50 == 0) cout << ".";
      cout.flush();
      
      StatsDB::const_iterator i = sdb.single_begin();
      while ( i != sdb.single_end() ) {
	result = sdb.statistic(*i,ds1);
	result = sdb.statistic(*i,ds2);
	++i;
      }
      
      i = sdb.double_begin();
      while ( i != sdb.double_end() ) {
	result = sdb.statistic(*i,ds1,ds2);
	++i;
      }
      ++k;
    }
    if (gettimeofday(&stop,0) == -1) cerr << "couldn't get time" << endl;
    
    // Roundabout to avoid overflowing long int.
    double fraction = (stop.tv_usec - start.tv_usec)/1000000.0;
    double whole  = stop.tv_sec - start.tv_sec;
    double elapsed   = whole + fraction;
    
    cout << " Done (" << elapsed << " seconds)." << endl;
    times[q-1] = elapsed;

    ++q;
  }
  // An extra line for good luck
  cout << endl;
  
  cout << "5 time tests, in 5,000 iteration increments: ";
  size_t w = 0;
  while (w < 5) {
    cout << times[w] << " ";
    ++w;
  }
  cout << " seconds." << endl;

  return 0;
}

// $Id: statsdb.cc,v 1.1 1998/09/29 23:16:53 hp Exp $
